# Configuration file for the general flow of managing the preprocessors
# and the prover-heuristic configuration.
import config_schedule # Import for specifying schedules

# Program reports the control flow when verbose is set to true.
# Otherwise it runs quietly until a solution is found
VERBOSE = False # {True | False}
#VERBOSE = True

# The prover executable
PROVER = "./res/iproveropt"

PROC_SLEEP_TIME = 0.5


# LTB params
LTB_PHASE_LOW_MAX_RUNTIME = 12
LTB_PHASE_HIGH_MIN_RUNTIME = 180


# When running internal schedules we need to specify the the prep heuristic as separate.
# To run on qbf preprocessing needs to contain at least "--qbf_mode true" to succesfully
# run. In this setup we run the best heuristic as the preprocessor. The best heuristics
# are in the single schedules, hence we create a mapping between them as it will be less to update.
INTERNAL_PREP_HEURISTICS = {"default": "heur/default",
                            "default_test": "heur/default",
                            "default_claus_test": "heur/default_claus_test",
                            "ueq_sched_no_abstr": "heur/245752"}


# Scheudle mapping for string and specification
#SCHEDULES = {"default": config_schedule.default,
#             "default_test": config_schedule.default_test,
#             "default_claus_test": config_schedule.default_claus_test,
#             "ueq_sched_no_abstr": config_schedule.ueq_sched_no_abstr}

SCHEDULES = {"ueq_sched_no_abstr": config_schedule.ueq_sched_no_abstr,
             "ltb_high_phase_schedule": config_schedule.ltb_high_phase_schedule,
             "fnt_schedule": config_schedule.fnt_schedule,
             "ltb_low_phase_v4": config_schedule.ltb_low_phase_v4,
             "ltb_low_phase_v5": config_schedule.ltb_low_phase_v5,
             "ltb_high_common": config_schedule.ltb_high_common,
             "fof_schedule": config_schedule.fof_schedule}

