import shutil
import os
import sys
import re


def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def create_tmp_dir():

    try:
        os.mkdir("tmp")
    except FileExistsError:
        pass


def clean_tmp_folder():
    # Clean tmp folder
    shutil.rmtree('tmp/')


def check_file_existence(file_path):

    # If a file doesn´t exists, exit on error
    if not os.path.exists(file_path):
        eprint("ERROR: file \"{0}\" does not exist".format(file_path))
        sys.exit(1)


def output_proof(proof_file):

    with open(proof_file, 'r', newline=None) as f:
        proof = f.read()

    print(proof)


def compute_timeout_bound(process_timeout, wc_timeout):

    if process_timeout is None:
        timeout = wc_timeout
    else:
        timeout = min(process_timeout, wc_timeout)

    return timeout


# Make test cases first!
# Count the number of clauses in a problem (including its axioms)
def count_no_clauses(problem_path):

    # Variable to store the number of fof clauses
    count = 0


    # Set in a try as we might not find the axiom file
    # (bit dependent on the file structure)
    try:
        # First open up the problem and count the clauses
        with open(problem_path, 'r') as f:
            data = f.read()

        # Get all the clauses in the problem
        count += len(re.findall("^fof", data, flags=re.MULTILINE))

        # See if there are any axiom files
        axioms = re.findall("^include.*", data, flags=re.MULTILINE)

        for axiom in axioms:
            # Get the axiom file
            ax_file = axiom[9:-3]

            # Get the path (a bit silly)
            dir_path = os.path.dirname(problem_path)
            dir_path = os.path.dirname(dir_path)
            ax_path = dir_path + "/" + ax_file

            # First open up the problem and count the clauses
            with open(ax_path, 'r') as f:
                data = f.read()

            # Get all the clauses in the problem
            count += len(re.findall("^fof", data, flags=re.MULTILINE))
    except Exception:
        pass


    return count
