#from iprover_process_module import start_prover_process, ProbProc, get_output_status, check_prover_success
#from scheduler import read_heuristic_string
import os
#from prover_runner import start_prover_external_schedule, get_prover_output_status, start_prover_internal_schedule
from prover_runner import build_prover_cmd_external, start_prover_process, get_prover_output_status, build_prover_cmd_internal
from helper import clean_tmp_folder, create_tmp_dir
from config import SCHEDULES


########################################################
# Check configs


# Extract the heuristic configs
heur_conf = []
for s in SCHEDULES.values():
    for h in s:
        heur_conf += [h]

# Check that every referenced heuristic exists
for conf in heur_conf:
    # if not os.path.isfile(conf[0]):
    if not os.path.isfile(conf[0]):
        print("Err heuristic {0} does not exist".format(conf[0]))

# Check that timing exists in a number or none
for conf in heur_conf:
    if len(conf) != 2:
        print("Err missing time for conf: {0}".format(conf))

    else:
        # Test if time is None
        if conf[1] is not None:
            try:
                # Test if number
                float(conf[1])
            except ValueError as err:
                print("Err conf {0}: ".format(conf), err)


######################################################
# Check contents of the heuristic strings

# Get the heuristic files
heuristics = [c[0] for c in heur_conf]
heuristics = sorted(set(heuristics))

# For every file
for heur in heuristics:

    # Get the contents of the file
    with open(heur, 'r') as f:
        d = f.read()

    # Make sure the string is not emtpy
    if d == "":
        print("Err heuristic {0} is empty".format(heur))
        continue

    # Check that proofs are suppressed (qbf)
    if "--res_out_proof false" in d:
        print("ERR heur {0}:  Suppressing resolution proof!".format(heur))
    if "--inst_out_proof false" in d:
        print("ERR heur {0}:  Suppressing instantiaton proof!".format(heur))
    if "--sat_out_model none" in d:
        print("ERR heur {0}:  Suppressing sat model!".format(heur))

    # Make sure timeout is removed from the heuristic
    if "--time_out_real" in d:
        print("ERR heur {0}: Timeout included!".format(heur))

    # Check that all heuristic has a clausifier set
    if "--clausifier res/vclausify_rel" not in d and "--clausifier res/eprover" not in d:
        print("ERR heur {0}: clausifier not set".format(heur))

######################################################
# Check that every heuristic in the path works!
# This part does not reveal all bugs

create_tmp_dir()
# Check that every heuriistic (and the caller) runs bug free on the prover

TIME = 3
#test_problem = "test_problems/floor256.qdimacs"
#test_problem = "test_problems/GRP564-1.p"
#test_problem = "test_problems/COL004-3.p"
test_problem = "test_problems/problem_fof.p"


# EXTERNAL heuristic test
for heur in heuristics:
    cmd_string, *_ = build_prover_cmd_external(test_problem, TIME, [heur, TIME])
    #print()
    #print(cmd_string)
    prover_proc = start_prover_process(cmd_string, TIME)
    _, err = prover_proc.proc.communicate(timeout=TIME + 12)
    output_file = prover_proc.output_file

    # Check that error is empty
    if err != b"" and err != b"\n\n":
        print("ERR running heuristic: {0} ".format(heur), err)

    # print(prover_out)
    # Check that problem had the expcted status
    # Read the proof output
    with open(output_file, 'r') as f:
        prover_out = f.read()
    szs_status = get_prover_output_status(prover_out)

    if szs_status != "Theorem": #"Unsatisfiable":
        print("Err incorrect SZS status \"{0}\" for heuristic {1}".format(szs_status, heur))


"""
# INERNAL  heuristic test
for heur in heuristics:

    # Build the schedule string
    sched_str = "[[{0};{1}]]".format(heur, TIME)

    cmd_string, *_ = build_prover_cmd_external(test_problem, TIME, [heur, TIME])
    prover_proc = start_prover_process(cmd_string, TIME)
    _, err = prover_proc.proc.communicate(timeout=TIME + 12)

    proc, output_file, *_ = start_prover_internal_schedule(test_problem, sched_str, TIME, heur)
    _, err = proc.communicate(timeout=TIME)

    # Check that error is empty
    if err != b"":
        print("ERR running INTERNAL heuristic: {0} ".format(heur), err)

    # print(prover_out)
    # Check that problem had the expcted status
    # Read the proof output
    with open(output_file, 'r') as f:
        prover_out = f.read()
    szs_status = get_prover_output_status(prover_out)

    if szs_status != "Satisfiable":
        print("Err incorrect SZS status \"{0}\" for heuristic {1}".format(szs_status, heur))
#"""

# Clean up tmp
clean_tmp_folder()
